/*
 * Block.cpp
 *
 *  Created on: 11.12.2015
 *      Author: Helge
 */

#include "Block.h"
#include "Index.h"
#include "FiberGraph.h"
#include "Global.h"

#include <algorithm>
#include <string>
#include <iostream>
#include <sstream>
#include <time.h>
#include <set>

BlockIndex::BlockIndex() {
	id = 0;
	counter = 0;
}

BlockIndex::BlockIndex(int _id) {
	id = _id;
	counter = 0;
}

BlockIndex::~BlockIndex() {

}

void BlockIndex::addBlock(intSet X, intSet Y, intSet Z) {
	//check if contained
	intSet xNeighbours;
	for (int x : X) {
		if ((MAP_CONTAINS(x, xindex))) {
			xNeighbours.insert(xindex[x].begin(),xindex[x].end());
		}
	}
	intSet yNeighbours;
	for (int x : Y) {
		if ((MAP_CONTAINS(x, yindex))) {
			yNeighbours.insert(yindex[x].begin(),yindex[x].end());
		}
	}
	intSet zNeighbours;
	for (int x : Z) {
		if ((MAP_CONTAINS(x, zindex))) {
			zNeighbours.insert(zindex[x].begin(),zindex[x].end());
		}
	}
	intSet xyCandidates;
	SET_INTERSECTION_TO_SET(xNeighbours,yNeighbours,xyCandidates)
	intSet candidates;
	SET_INTERSECTION_TO_SET(zNeighbours,xyCandidates,candidates)
	for (int bid : candidates) {
		if ((coords[bid]->X == X) && (coords[bid]->Y == Y) && (coords[bid]->Z == Z)) {
			//Duplicate
			return;
		}
	}


	// Add the block
	blockIds.insert(counter);
	coords.push_back(new Block(X, Y, Z));
	for (int x : X) {
		if (!(MAP_CONTAINS(x, xindex))) {
			xindex[x] = intSet();
		}
		xindex[x].insert(counter);
	}
	for (int y : Y) {
		if (!(MAP_CONTAINS(y, yindex))) {
			yindex[y] = intSet();
		}
		yindex[y].insert(counter);
	}
	for (int z : Z) {
		if (!(MAP_CONTAINS(z, zindex))) {
			zindex[z] = intSet();
		}
		zindex[z].insert(counter);
	}
	counter += 1;
}

std::vector<Block*> BlockIndex::getBlocks() {
	return coords;
}

void BlockIndex::addBlockSingleton(int x, int y, int z) {
	intSet X = *(new intSet());
	X.insert(x);
	intSet Y = *(new intSet());
	Y.insert(y);
	intSet Z = *(new intSet());
	Z.insert(z);
	blockIds.insert(counter);
	Block* nBlock = new Block(X, Y, Z);
	coords.push_back(nBlock);
	counter += 1;
}

//TODO no copy
void BlockIndex::mergeBlocks(int leftblock, int rightblock) {
	blockIds.erase(rightblock);
//	intSet lX = coords[leftblock].X;
//	intSet lY = coords[leftblock].Y;
//	intSet lZ = coords[leftblock].Z;
	intSet rX = coords[rightblock]->X;
	intSet rY = coords[rightblock]->Y;
	intSet rZ = coords[rightblock]->Z;
	coords[leftblock]->X.insert(rX.begin(), rX.end());
	coords[leftblock]->Y.insert(rY.begin(), rY.end());
	coords[leftblock]->Z.insert(rZ.begin(), rZ.end());

}

void BlockIndex::mergeXYZ(int leftblock, intSet X, intSet Y, intSet Z) {
	//(lX,lY,lZ) = self.coords[leftblock]
	//self.coords[leftblock] = (lX.union(X),lY.union(Y),lZ.union(Z))
	coords[leftblock]->X.insert(X.begin(), X.end());
	coords[leftblock]->Y.insert(Y.begin(), Y.end());
	coords[leftblock]->Z.insert(Z.begin(), Z.end());
}

bool BlockIndex::update(std::vector<coordinate>* coords, BloomFilter* bfMask) {
	intSet X;
	intSet Y;
	intSet Z;
	for (coordinate coord : *coords) {
		X.insert(coord.x);
		Y.insert(coord.y);
		Z.insert(coord.z);
	}
	if ((X.size() < global::BLOCK_MINSIZE) || (Y.size() < global::BLOCK_MINSIZE)
			|| (Z.size() < global::BLOCK_MINSIZE)) {
		return false;
	}
	if ((X.size() * Y.size() * Z.size()) < global::BLOCK_MINVOL) {
		return false;
	}
	bool freq = false;
#pragma omp critical
	{
		freq = bfMask->checkDensity(X, Y, Z, global::BLOCK_THRESHOLD);
		if (freq) {
//		LOG_DB(
//				"Block: X = " + toString(X) + " | Y = " + toString(Y)
//						+ " | Z = " + toString(Z))
			addBlock(X, Y, Z);
			for (int x : X) {
				for (int y : Y) {
					for (int z : Z) {
						bfMask->add(x, y, z);
					}
				}
			}
		}
	}
	return freq;
}

bool BlockIndex::update(std::vector<coordinate>* coords, intVector* candidates,
		BloomFilter* bfMask) {
	intSet X;
	intSet Y;
	intSet Z;
	for (int candidateID : *candidates) {
		coordinate coord = (*coords)[candidateID];
		X.insert(coord.x);
		Y.insert(coord.y);
		Z.insert(coord.z);
	}
	if ((X.size() < global::BLOCK_MINSIZE) || (Y.size() < global::BLOCK_MINSIZE)
			|| (Z.size() < global::BLOCK_MINSIZE)) {
		return false;
	}
	if ((X.size() * Y.size() * Z.size()) < global::BLOCK_MINVOL) {
		return false;
	}
	bool freq = false;
//#pragma omp critical
//	{
		freq = bfMask->checkDensity(X, Y, Z, global::BLOCK_THRESHOLD);
		if (freq) {
			//INC_INT_STAT("WALK_NUM_BLOCKS")
//		LOG_DB(
//				"Block: X = " + toString(X) + " | Y = " + toString(Y)
//						+ " | Z = " + toString(Z))
			addBlock(X, Y, Z);
			bfMask->addBlock(X, Y, Z);
//			for (int x : X) {
//				for (int y : Y) {
//					for (int z : Z) {
//						bfMask->add(x, y, z);
//					}
//				}
//			}
		}
//	}
	return freq;
}

void BlockIndex::toFile(std::string fileName) {
	std::ofstream blockFile;
	blockFile.open(fileName.c_str());
	//std::vector<Block*> blocks = blockIndex->coords;
	for (Block* block : coords) {
		int tp = global::BFmask->checkDensityTp(block->X,block->Y,block->Z);
		int size = block->X.size() * block->Y.size() * block->Z.size();
//		LOG_DB(
//				"Block(" +  toString(((float) tp) / ((float) size)) + "): X = " + toString(block->X) + " | Y = "
//						+ toString(block->Y) + " | Z = " + toString(block->Z))
		for (int x : block->X) {
			blockFile << toString(x) << "\t";
		}
		blockFile << "\n";
		for (int x : block->Y) {
			blockFile << toString(x) << "\t";
		}
		blockFile << "\n";
		for (int x : block->Z) {
			blockFile << toString(x) << "\t";
		}
		blockFile << "\n\n";
	}
	blockFile.close();
}

