/*
 * Graph.cpp
 *
 *  Created on: 14.12.2015
 *      Author: Helge
 */

#include "FiberGraph.h"
#include "Global.h"

#include <algorithm>
#include <fstream>
#include <sstream>
#include <iostream>
#include <string>
#include <iterator>
#include <map>
#include <unordered_set>
#include <vector>

using namespace std;

FiberGraph::FiberGraph(std::vector<coordinate>* in_coords) {
	int size = in_coords->size();
	numNodes = 0;
	neighborsbyId.resize(size);
	coordinates = in_coords;
	addAll(coordinates);
}

FiberGraph::~FiberGraph() {
}

intSet* FiberGraph::getNeighbours(coordMapPtr* map, intPair key) {
	intSet* n = NULL;
	if (EX_MAP_CONTAINS(key, map)) {
		n = (*map)[key];
	}
	return n;
}

void FiberGraph::toMap(coordMapPtr* map, intPair key, int id) {
	if (!(EX_MAP_CONTAINS(key, map))) {
		(*map)[key] = new intSet();
		(*map)[key]->insert(id);
	} else {
		(*map)[key]->insert(id);
	}
}

void FiberGraph::addAll(std::vector<coordinate>* coords) {
	/*find Independent tensor entries = entries with no common coordinates*/
	for (int coordId = 0; coordId < coords->size(); coordId++) {
		int x = (*coords)[coordId].x;
		int y = (*coords)[coordId].y;
		int z = (*coords)[coordId].z;
		add(x, y, z);
	}
}

void FiberGraph::add(int x, int y, int z) {
	if (neighborsbyId.size() < numNodes) {
		throw std::runtime_error("Internal: Initial size was too small!");
	}

	intSet* neighbours = new intSet();
	int maxId = numNodes;
	//What exisitng nodes are neighbours?
	intSet* Xneighbours = getNeighbours(&Xmap, intPair(y, z));
	intSet* Yneighbours = getNeighbours(&Ymap, intPair(x, z));
	intSet* Zneighbours = getNeighbours(&Zmap, intPair(x, y));
	if (Xneighbours != NULL)
		neighbours->insert(Xneighbours->begin(), Xneighbours->end());
	if (Yneighbours != NULL)
		neighbours->insert(Yneighbours->begin(), Yneighbours->end());
	if (Zneighbours != NULL)
		neighbours->insert(Zneighbours->begin(), Zneighbours->end());
	for (auto n : *neighbours) {
		//Add me to my neighbours
		neighborsbyId[n]->insert(maxId);
	}
	// Turn neighbours into a list
	neighborsbyId[maxId] = neighbours;
	//add me to X,Y and Z map
	toMap(&Xmap, intPair(y, z), maxId);
	toMap(&Ymap, intPair(x, z), maxId);
	toMap(&Zmap, intPair(x, y), maxId);
	//Increase counter
	numNodes++;
}

const intSet* FiberGraph::getNeighbours(int node) {
	return neighborsbyId[node];
}

SPLIT_RESULT FiberGraph::split() {
	int iter = 0;
	int startNode = rand() % numNodes;
	int maxDis = 0;
	intVector goalNodes;
	goalNodes.push_back(startNode);
	bool nextIteration = true;
	int INF_DIS = numNodes;

	intVector disFromStart(numNodes);
	while (nextIteration) {
		iter++;
		nextIteration = false;
		for (int i = 0; i < numNodes; i++)
			disFromStart[i] = INF_DIS;
		disFromStart[startNode] = 0;
		std::queue<int> openNodes;
		openNodes.push(startNode);
		while (openNodes.size() > 0) {
			int curNode = openNodes.front();
			openNodes.pop();
			int curDistance = disFromStart[curNode];

			const intSet* neighbours = getNeighbours(curNode);
			for (int neighId : *neighbours) {
				int neighDistance = disFromStart[neighId];
				if (neighDistance > curDistance + 1) {
					disFromStart[neighId] = curDistance + 1;
					openNodes.push(neighId);
				} else if (curDistance > neighDistance + 1) {
					disFromStart[curNode] = neighDistance + 1;
					openNodes.push(curNode);
				}
			}
		}
		int newMaxDis = 0;
		for (int j = 0; j < numNodes; j++) {
			if (disFromStart[j] > newMaxDis && disFromStart[j] != INF_DIS) {
				newMaxDis = disFromStart[j];
			}
		}
		goalNodes.resize(0);
		for (int i = 0; i < numNodes; i++) {
			if (disFromStart[i] == newMaxDis) {
				goalNodes.push_back(i);
			}
		}
		if (newMaxDis > maxDis) {
			maxDis = newMaxDis;
			startNode = goalNodes[rand() % goalNodes.size()];
			nextIteration = true;
		} else {
			nextIteration = false;
		}
	}
	int goalNode = goalNodes[rand() % goalNodes.size()];
//	LOG_DB(
//			"max dis = " + toString(maxDis) + " in " + toString(iter)
//					+ " iterations. from " + toString(startNode) + " to "
//					+ toString(goalNode))

	intVector disFromGoal(numNodes);
	for (int i = 0; i < numNodes; i++)
		disFromGoal[i] = INF_DIS;
	disFromGoal[goalNode] = 0;
	std::queue<int> openNodes;
	openNodes.push(goalNode);
	while (openNodes.size() > 0) {
		int curNode = openNodes.front();
		openNodes.pop();
		int curDistance = disFromGoal[curNode];

		const intSet* neighbours = getNeighbours(curNode);
		for (int neighId : *neighbours) {
			int neighDistance = disFromGoal[neighId];
			if (neighDistance > curDistance + 1) {
				disFromGoal[neighId] = curDistance + 1;
				openNodes.push(neighId);
			} else if (curDistance > neighDistance + 1) {
				disFromGoal[curNode] = neighDistance + 1;
				openNodes.push(curNode);
			}
		}
	}

	intVector* goalPart = new intVector();
	intVector* startPart = new intVector();

	for (int i = 0; i < numNodes; i++) {
		if (disFromStart[i] == INF_DIS || disFromGoal[i] == INF_DIS) {
//			LOG_WAR("inf dis!")
			continue;
		}
		if (disFromStart[i] <= disFromGoal[i]) {
			startPart->push_back(i);
		} else {
			goalPart->push_back(i);
		}
//		if (disFromGoal[i] < max_distance && disFromStart[i] < max_distance) {
//			continue;
//		}
//		if (disFromGoal[i] < disFromStart[i]) {
//			if (disFromGoal[i] != INF_DIS && disFromGoal[i] >= max_distance)
//				goalPart->push_back(i);
//		} else {
//			if (disFromStart[i] != INF_DIS && disFromStart[i] >= max_distance)
//				startPart->push_back(i);
//		}
	}
	//if (startPart->)
//	LOG_DB("startPart = " + toString(*startPart));
//	LOG_DB("goalPart = " + toString(*goalPart));

	SPLIT_RESULT result;
	result.first = startPart;
	result.second = goalPart;
	result.diameter = maxDis;
	return result;
}

void FiberGraph::printGraph() {
//LOG_DB(toString(neighborsbyId))
	for (int node = 0; node < numNodes; node++) {
		LOG_DB(toString(node) + " = " + toString(*neighborsbyId[node]))
	}
}

void FiberGraph::printGraphviz() {
	std::cout << "graph FG {" << std::endl;
	for (int node = 0; node < numNodes; node++) {
		for (int neighNode : *neighborsbyId[node]) {
			if (node > neighNode)
				std::cout
						<< "node" + toString(node) + " -- node"
								+ toString(neighNode) << std::endl;
		}
		std::cout
				<< "node" + toString(node) + " [label = \""
						+ toString((*coordinates)[node]) + "\"]" << std::endl;
	}
	std::cout << "}" << std::endl;
}

void FiberGraph::toFile(std::string fileName) {
	std::ofstream* graphFile = new std::ofstream(fileName);
	for (int node = 0; node < numNodes; node++) {
		for (int neighNode : *neighborsbyId[node]) {
			if (node > neighNode)
				*graphFile << toString(node) + " " + toString(neighNode) + " 1"
						<< std::endl;
		}
	}
	graphFile->close();
}

FiberGraph * createPartitonGraph(std::vector<coordinate> *coordinates,
		intVector * candidates) {
	std::vector<coordinate>* graph_coordinates = new std::vector<coordinate>(
			candidates->size());
	int pos = 0;
	for (int candId : *candidates) {
		if (pos > candidates->size()) {
			LOG_WAR(" candidates size is strange... ")
		}
		(*graph_coordinates)[pos] = (*coordinates)[candId];
		pos++;
	}
	return new FiberGraph(graph_coordinates);
}

