/*
 * Graph.h
 *
 *  Created on: 14.12.2015
 *      Author: Helge
 */

#ifndef FIBERGRAPH_H_
#define FIBERGRAPH_H_

#include <string>
#include <vector>
#include <set>
#include <map>
#include "Utility.h"

typedef std::map<intPair, intSet> coordMap;
typedef std::map<intPair, intSet*> coordMapPtr;

struct SPLIT_RESULT {
	intVector* first;
	intVector* second;
	int diameter;
};

FiberGraph * createPartitonGraph(std::vector<coordinate> *coordinates,
		intVector * candidates);

class FiberGraph {
private:
	void addAll(std::vector<coordinate>* coords);
	void add(int x, int y, int z);
	intSet* getNeighbours(coordMapPtr* map, intPair key);
	void toMap(coordMapPtr* map, intPair key, int id);
public:
	int numNodes;
	std::vector<intSet*> neighborsbyId;
	std::vector<coordinate>* coordinates;
	coordMapPtr Xmap; // maps <y,z> to id list
	coordMapPtr Ymap; // maps <x,z> to id list
	coordMapPtr Zmap; // maps <x,y> to id list
	FiberGraph(std::vector<coordinate>* coords);

	/* Destructor */
	~FiberGraph();


	/* get All Neighbours of a node in the current Graph, deleted Nodes return NULL.
	 * This NULL states should only happen internally during deletion */
	const intSet* getNeighbours(int nodeId);

	SPLIT_RESULT split();

	/*Prints an list representation of the Graph on the Debug channel*/
	void printGraph();
	void printGraphviz();
	void toFile(std::string fileName);
};

#endif /* FIBERGRAPH_H_ */
