/*
 * Global.h
 *
 *  Created on: 15.12.2015
 *      Author: Helge
 */

#ifndef GLOBAL_H_
#define GLOBAL_H_

//#include "Utility.h"
#include <set>
#include <map>
#include <vector>
#include <string>
#include <unordered_set>
#include <time.h>

class BloomFilter;
class FiberGraph;
class BlockIndex;
class Tensor;

enum LOG_LEVEL {
	MY_ERROR, MY_WARNING, MY_INFO, MY_DEBUG, MY_TRACE
};

// ==== Internal Settings
#define IS_DEBUG false
#define IS_TRACE false
#define TEST_SANITY false
#define SANITY_CHECK(cond) if (TEST_SANITY && (!(cond))) {throw std::runtime_error("Sanity Check failed!");}

// -- TYPEDEFS
typedef std::pair<int, int> intPair;
typedef std::pair<std::string, std::string> stringPair;
typedef std::set<int> intSet;
typedef std::unordered_set<int> intHashSet;
typedef std::vector<int> intVector;
typedef std::vector<intVector> intVectorVector;
typedef std::map<int, std::map<int, intSet*>*> CoordIndexMap;

// ==== TYPES
// -- CLASSES / STRUCTS
struct coordinate {
	coordinate();
	coordinate(int in_x, int in_y, int in_z);
	void update(int in_x, int in_y, int in_z);
	int x;
	int y;
	int z;
};

struct coordinateSet {
	coordinateSet(intSet in_X, intSet in_Y, intSet in_Z) {
		X = in_X;
		Y = in_Y;
		Z = in_Z;
	}
	intSet X;
	intSet Y;
	intSet Z;
};

// ==== Constants
template<typename T>
struct Array {
	const T *entries;
	const int numEntries;
};

namespace global {

// CONSTANTS
extern std::string CONFIG_FILE;

// SETTINGS
extern std::string INPUT_FILE;
#define INPUT_FILE_DEFAULT ""
extern std::string OUTPUT_FILE;
#define OUTPUT_FILE_DEFAULT "blocks.txt"
extern std::string REPORT_FILE;
#define REPORT_FILE_DEFAULT "report.txt"

extern std::vector<coordinate> coordinates;
extern BloomFilter* BF;
extern BloomFilter* BFmask;
extern FiberGraph* Graph;

//Parameters
extern int BLOCK_MINSIZE;
#define BLOCK_MINSIZE_DEFAULT 2
extern int BLOCK_MINVOL;
#define BLOCK_MINVOL_DEFAULT 8
extern float BLOCK_THRESHOLD;
#define BLOCK_THRESHOLD_DEFAULT 0.9

extern int EXPAND_NUM_SEEDS;
#define EXPAND_NUM_SEEDS_DEFAULT 5
extern int SPLIT_MIN_DIAM;
#define SPLIT_MIN_DIM_DEFAULT 3;

extern long BLOOM_CAPACITY;
#define BLOOM_CAPACITY_DEFAULT 10000000
extern float BLOOM_ERROR_RATE;
#define BLOOM_ERROR_RATE_DEFAULT 0.01;

//Properties
extern int MAX_X;
extern int MAX_Y;
extern int MAX_Z;
extern int SIZE_TENSOR;

//functions
void reset();
std::string allToString();

}  // namespace global

#endif /* GLOBAL_H_ */
