/*
 * Filter.cpp
 *
 *  Created on: 07.01.2016
 *      Author: Helge
 */

#include "Index.h"
#include "Block.h"
#include "FiberGraph.h"
#include "Global.h"

#include <map>
#include <math.h>       /* floor */

//A Bloom filter implemented in C
BloomFilter::BloomFilter(int _capacity, double _errorRate) {
	capacity = _capacity;
	errorRate = _errorRate;
	bf = bloom_create_mbf(capacity, errorRate);
	if (bf == NULL) {
		throw std::runtime_error("mbf is NULL!");
	}
}

std::string BloomFilter::to_string() {
	throw std::runtime_error("Not yet implemented!");
}

void BloomFilter::add(int key1, int key2, int key3) {
	numAdded++;

	if (IS_DEBUG)
		if (numAdded > capacity) {
			LOG_WAR("Capacity too small! + (" + toString(numAdded) + "/" + toString(capacity));
		}
	bloom_add_mbf(bf, key1, key2, key3);
}

//adds whole block (product of vectors X,Y,Z) to BF
void BloomFilter::addBlock(intSet X, intSet Y, intSet Z) {
	for (int key1 : X) {
		for (int key2 : Y) {
			for (int key3 : Z) {
				numAdded++;
				bloom_add_mbf(bf, key1, key2, key3);
			}
		}
	}
//	LOG_DB("NumAdded = " + toString(numAdded));
}

//checks if cell (key1,key2,key3) is covered
int BloomFilter::check(int key1, int key2, int key3) {
	return bloom_check_mbf(bf, key1, key2, key3);
}

//Checks how many 1s there are in all the combinations of x, y, and z.
int BloomFilter::check_ones_product(intSet x, intSet y, intSet z, int minones) {
	//check_ones_product();
	int* x_a = setToArr(x);
	int* y_a = setToArr(y);
	int* z_a = setToArr(z);
	return bloom_check_ones_product(bf, x_a, x.size(), y_a, y.size(), z_a,
			z.size(), minones);
}

//counts how many cells in key1 x key2 x key3 are covered. returns num of zeros or maxErr if larger than that
int BloomFilter::multi_check_zeros(int key1, int key2, int key3, int vecLen,
		int maxErr) {
	throw std::runtime_error("Not yet implemented!");
}

//Checks how many elements in the merge are NOT covered
int BloomFilter::check_merge(intSet l0, intSet l1, intSet l2, intSet t0,
		intSet t1, intSet t2, int maxErr) {
	int* a0 = setToArr(l0);
	int* a1 = setToArr(l1);
	int* a2 = setToArr(l2);
	int* b0 = setToArr(t0);
	int* b1 = setToArr(t1);
	int* b2 = setToArr(t2);
	int* la = new int[3];
	la[0] = l0.size();
	la[1] = l1.size();
	la[2] = l2.size();
	int* lb = new int[3];
	lb[0] = t0.size();
	lb[1] = t1.size();
	lb[2] = t2.size();
	return bloom_check_merge_mbf(bf, a0, a1, a2, la, b0, b1, b2, lb, maxErr);
}

bool BloomFilter::checkDensity(intSet X, intSet Y, intSet Z, float thres) {
	int size = X.size() * Y.size() * Z.size();
	int tp = check_ones_product(X, Y, Z, (int) (floor(size * thres)));
	if (tp >= (int) (floor(size * thres))) {
		return true;
	}
	return false;
}

int BloomFilter::checkDensityTp(intSet X, intSet Y, intSet Z) {
	int size = X.size() * Y.size() * Z.size();
	return check_ones_product(X, Y, Z, size);
}

CoordIndexMap* createCoordIndexMap(std::vector<coordinate>* coords) {
	CoordIndexMap* IndexMap = new CoordIndexMap();
	for (coordinate coord : *coords) {
		int x = coord.x;
		int y = coord.y;
		int z = coord.z;
		if (!(EX_MAP_CONTAINS(x, IndexMap))) {
			(*IndexMap)[x] = new std::map<int, intSet*>();
		}
		if (!(EX_MAP_CONTAINS(y, (*IndexMap)[x]))) {
			intSet* y_ptr = new intSet();
			(*(*IndexMap)[x])[y] = y_ptr;
		}
		(*(*IndexMap)[x])[y]->insert(z);
	}
	return IndexMap;
}

//TODO use handler, add erase, don't use this function right now? or do? no idea...
void destroyCoordIndexMap(CoordIndexMap* coordMap) {
	for (auto entry1 : *coordMap) {
		auto entry1Map = entry1.second;
		for (auto entry2 : *entry1Map) {
			auto entry2set = entry2.second;
			delete entry2set;
		}
		delete entry1Map;
	}
	delete coordMap;
}
