/*
 * Filter.h
 *
 *  Created on: 07.01.2016
 *      Author: Helge
 */

#ifndef FILTER_H_
#define FILTER_H_

#include "Utility.h"
#include "pBloomFilter.h"

class BloomFilter {
private:
	int capacity;
	int numAdded = 0;
	double errorRate;
	std::string bitExpToBytes(int x);
public:
	mbf * bf;

	//A Bloom filter implemented in C
	BloomFilter(int _capacity, double _errorRate);
	std::string to_string();
	void add(int key1, int key2, int key3);
	//adds whole block (product of vectors X,Y,Z) to BF
	void addBlock(intSet X, intSet Y, intSet Z);
	//checks if cell (key1,key2,key3) is covered
	int check(int key1, int key2, int key3);
	//counts how many cells in key1 x key2 x key3 are covered. returns num of zeros or maxErr if larger than that
	int multi_check_zeros(int key1, int key2, int key3, int vecLen, int maxErr);
	//Checks how many 1s there are in all the combinations of x, y, and z.
	int check_ones_product(intSet x,intSet y,intSet z,int minones);
	//Checks how many elements in the merge are NOT covered
	int check_merge(intSet l0,intSet l1,intSet l2,intSet t0,intSet t1,intSet t2,int maxErr);
//	float BloomFilter::check_merge_fast(intSet l0,intSet l1,intSet l2,intSet t0,intSet t1,intSet t2,int maxErr){

	bool checkDensity(intSet X,intSet Y,intSet Z,float thres);
	int checkDensityTp(intSet X,intSet Y,intSet Z);
};

//bool checkDensity(BloomFilter* bfMask,intSet X,intSet Y,intSet Z,float thres);

CoordIndexMap* createCoordIndexMap(std::vector<coordinate>* coords);

#endif /* FILTER_H_ */
