/*
 * Utility.cpp
 *
 *  Created on: 14.12.2015
 *      Author: Helge
 */

/*
 * Utility
 *
 *  Created on: 03.12.2015
 *      Author: Helge
 */

#include "Utility.h"
#include "Global.h"
//#include "MemoryUsage.h"

#include <string>
#include <iostream>
#include <sstream>
#include <time.h>
#include <set>
#include <iomanip>
#include <algorithm>

using namespace std;

intSet emptySet = intSet();

// ==== TYPES
coordinate::coordinate() {
	x = -1;
	y = -1;
	z = -1;
}
coordinate::coordinate(int in_x, int in_y, int in_z) {
	x = in_x;
	y = in_y;
	z = in_z;
}
void coordinate::update(int in_x, int in_y, int in_z) {
	x = in_x;
	y = in_y;
	z = in_z;
}

std::string padStringAfter(std::string toPad, int length) {
	std::ostringstream os;
	os << toPad << std::setw(length);
	return os.str();
}

const string currentDateTime() {
	time_t now = time(0);
	struct tm tstruct;
	char buf[80];
	tstruct = *localtime(&now);
	// Visit http://en.cppreference.com/w/cpp/chrono/c/strftime
	strftime(buf, sizeof(buf), "%Y-%m-%d.%X", &tstruct);
	return buf;
}

std::vector<int>* rangeVec(const int size) {
	vector<int> *result = new vector<int>(size);
	std::iota(result->begin(), result->end(), 0);
	return result;
}

std::set<int>* rangeSet(const int size) {
	return vecToSet(*rangeVec(size));
}

/**
 * Converts a intSet to a intVector with the same content
 * @return the vector */
std::vector<int>* setToVec(const std::set<int> in_set) {
	//TODO use build in functions
	std::vector<int>* result = new std::vector<int>();
	for (int elem : in_set) {
		result->push_back(elem);
	}
	return result;
}

/**
 * Converts a unordered intSet to a intVector with the same content
 * @return the vector */
std::vector<int>* setToVec(const std::unordered_set<int> in_set) {
	return new std::vector<int>(in_set.begin(), in_set.end());
}

/**
 * Converts a intVector to a set with the same content
 * @return the vector */
std::set<int>* vecToSet(const std::vector<int> in_vec) {
	return new std::set<int>(in_vec.begin(), in_vec.end());
}

/**
 * Converts a intVector to a set with the same content
 * @return the vector */
std::set<int>* vecToSet(const std::vector<int>* in_vec) {
	return new std::set<int>(in_vec->begin(), in_vec->end());
}

int* setToArr(intSet in_set) {
	int* result = new int[in_set.size()];
	int i = 0;
	for (int elem : in_set) {
		result[i] = elem;
		i++;
	}
	return result;
}

int* vecToArr(intSet in_vec) {
	int* result = new int[in_vec.size()];
	int i = 0;
	for (int elem : in_vec) {
		result[i] = elem;
		i++;
	}
	return result;
}

/**
 * Selects Randomly from an iterator
 * @Return the chosen element */
template<typename Iter, typename RandomGenerator>
Iter selectRandomly(Iter start, Iter end, RandomGenerator& g) {
	std::uniform_int_distribution<> dis(0, std::distance(start, end) - 1);
	std::advance(start, dis(g));
	return start;
}

/**
 * Selects Randomly from an iterator
 * @Return the chosen element */
template<typename Iter>
Iter selectRandomly(Iter start, Iter end) {
	static std::random_device rd;
	static std::mt19937 gen(rd());
	return selectRandomly(start, end, gen);
}

/**
 * Selects Randomly from an intSet
 * @Return the chosen element */
int selectRandomly(intSet set) {
	return *selectRandomly(set.begin(), set.end());
}

/**
 * Selects Randomly from an intSet
 * @Return the chosen element */
int selectRandomly(unordered_set<int> set) {
	return *selectRandomly(set.begin(), set.end());
}

/**
 * Selects Randomly from an intSet ptr
 * @Return the chosen element */
int selectRandomly(const intSet* set) {
	return *selectRandomly(set->begin(), set->end());
}

/**
 * Selects Randomly from an intVector
 * @Return the chosen element */
int selectRandomly(intVector vec) {
	int randomIndex = rand() % vec.size();
	return vec[randomIndex];
}

/**
 * Selects uniformly at random from a range of two ints
 * @Return the chosen int */
int randomFromRange(int lower, int upper) {
	if (TEST_SANITY) {
		if (upper < lower) {
			throw std::runtime_error(
					"Upper bound smaller than lower bound! (" + toString(lower)
							+ " < " + toString(upper));
		}
	}
	return lower + (rand() % (upper - (lower - 1)));
}

/**
 * Selects uniformly at random from a range of two double
 * @Return the chosen double */
double selectFromRangeContinious(double lower, double upper) {
	if (TEST_SANITY) {
		if (upper < lower) {
			throw std::runtime_error(
					"Upper bound smaller than lower bound! (" + toString(lower)
							+ " < " + toString(upper));
		}
	}
	double zeroOne = (double) rand() / (double) RAND_MAX;
	return lower + (zeroOne * (upper - lower));
}

/**
 * Selects a random number between 0.0 and 1.0
 * @Return the chosen double */
double selectFromRangeContinious() {
	return selectFromRangeContinious(0.0, 1.0);
}

// System Level Helper Functions
//TODO implement them
bool folderExsits(std::string path) {
//	struct stat info;
//
//	if (stat(path.c_str(), &info) != 0)
//		return false;
////	    printf( "cannot access %s\n", pathname );
//	else if (info.st_mode & S_IFDIR)  // S_ISDIR() doesn't exist on my windows
////	    printf( "%s is a directory\n", pathname );
//		return true;
//	else
////	    printf( "%s is no directory\n", pathname );
		return false;
}
