/*
 * Utility
 *
 *  Created on: 03.12.2015
 *      Author: Helge
 */

#ifndef UTILITY_
#define UTILITY_

#include "Global.h"
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <stdexcept>
#include <stdio.h>
#include <set>
#include <unordered_set>
#include <map>
#include <vector>
#include <queue>
#include <limits>
#include <random>
#include <exception>
#include <algorithm>
#include <time.h>

// ================
// ==== MACROS ====
// ================
//------------------------------------------------------------------------------------------------------

#define LOG_INFO(msg) std::cout <<              "          " << msg << std::endl;
#define LOG_ERR(msg) std::cout <<               "[ERROR]   " << msg << "!!!" << std::endl;
#define LOG_WAR(msg) std::cout <<               "[WARNING] " << msg << "!" << std::endl;
#define LOG_DB(msg) if (IS_DEBUG) {std::cout << "[DEBUG]   " << msg << std::endl;}
#define LOG_TR(msg) if (IS_TRACE) {std::cout << "   [TR]   " << msg << std::endl;}
#define LOG_TIME(name) LOG_INFO(std::string("[Time Stamp]") + name +std::string(" : ") + toString(clock())); setTimeStamp(name);
#define GET_TIME(name) getTimeStamp(name)

#define MAP_CONTAINS(key,map) (map.find(key) != map.end())
#define EX_MAP_CONTAINS(key,map) (map->find(key) != map->end())
#define SET_CONTAINS(key,set) (set.find(key) != set.end())
#define EX_SET_CONTAINS(key,set) (set->find(key) != set->end())

#define SET_INTERSECTION_TO_SET(first,second,intersect) std::set_intersection (first.begin(), first.end(), second.begin(), second.end(),std::inserter(intersect,intersect.begin()));
#define EX_SET_INTERSECTION_TO_SET(first,second,intersect) std::set_intersection (first->begin(), first->end(), second->begin(), second->end(),std::inserter(intersect,intersect.begin()));
#define SET_INTERSECTION_TO_VEC(first,second,intersect) std::set_intersection (first.begin(), first.end(), second.begin(), second.end(),std::back_inserter(intersect));
#define SET_UNION_TO_SET(first,second,intersect) std::set_union (first.begin(), first.end(), second.begin(), second.end(),std::inserter(intersect,intersect.begin()));
#define SET_UNION_TO_VEC(first,second,intersect) std::set_union (first.begin(), first.end(), second.begin(), second.end(),std::back_inserter(intersect));

#define ELAPSED_TIME(start,end) (float( end - start ) /  CLOCKS_PER_SEC)

// ===================
// ==== CONSTANTS ====
// ===================
//------------------------------------------------------------------------------------------------------

const int MIN_INT = std::numeric_limits<int>::min();
const int MAX_INT = std::numeric_limits<int>::max();
const long MIN_LONG = std::numeric_limits<long>::min();
const long MAX_LONG = std::numeric_limits<long>::max();
const long long MIN_LONG_LONG = std::numeric_limits<long long>::min();
const long long MAX_LONG_LONG = std::numeric_limits<long long>::max();

// ==============================
// ==== GENERATION FUNCTIONS ====
// ==============================
//------------------------------------------------------------------------------------------------------

/**
 * Generates a vector with the natural numbers from 0 to (size-1)
 * @return the vector */
std::vector<int>* rangeVec(const int size);

/**
 * Generates a set with the natural numbers from 0 to (size-1)
 * @return the vector */
std::set<int>* rangeSet(const int size);

// ========================
// ==== CAST FUNCTIONS ====
// ========================
//------------------------------------------------------------------------------------------------------

/**
 * Converts a set of int to a vectorwith the same content
 * @return the vector */
std::vector<int>* setToVec(const std::set<int> in_set);
std::vector<int>* setToVec(const std::unordered_set<int> in_set);

/**
 * Converts a set of int to a vectorwith the same content
 * @return the vector */
std::set<int>* vecToSet(const std::vector<int> in_vec);
std::set<int>* vecToSet(const std::vector<int>* in_vec);

int* setToArr(intSet in_set);
int* vecToArr(intSet in_vec);

// ====================================
// ==== RANDOM SELECTION FUNCTIONS ====
// ====================================
//------------------------------------------------------------------------------------------------------

template<typename Iter, typename RandomGenerator>
Iter selectRandomly(Iter start, Iter end, RandomGenerator& g);

/**
 * Selects a random element from an iterator
 * @return random element from iterator*/
template<typename Iter>
Iter selectRandomly(Iter start, Iter end);

/**
 * Selects a random element from a set
 * @return random element from the input set */
int selectRandomly(intSet set);
int selectRandomly(std::unordered_set<int> set);
int selectRandomly(const intSet* set);

/**
 * Selects a random element from a vector
 * @return random element from the input vector */
int selectRandomly(intVector vec);

/**
 * Selects a element x randomly from a range (s.t. lower <= x <= upper)
 * @return the random element */
int randomFromRange(int lower, int upper);

double randomFromRangeContinious(int lower, int upper);

double selectFromRangeContinious();

// ==================================
// ==== MAP COMPONENTS FUNCTIONS ====
// ==================================
//------------------------------------------------------------------------------------------------------

struct RetrieveKey {
	template<typename T>
	typename T::first_type operator()(T keyValuePair) const {
		return keyValuePair.first;
	}
};

struct RetrieveVal {
	template<typename T>
	typename T::first_type operator()(T keyValuePair) const {
		return keyValuePair.second;
	}
};

template<typename keyT, typename valT>
std::vector<keyT> getKeySetasVec(std::map<keyT, valT> in_map) {
	std::vector<keyT> keys;
	std::transform(in_map.begin(), in_map.end(), std::back_inserter(keys),
			RetrieveKey());
	return keys;
}

template<typename keyT, typename valT>
std::set<keyT> getKeySet(std::map<keyT, valT> in_map) {
	std::set<keyT> keys;
	for (auto entry : in_map) {
		keys.insert(entry.first);
	}
	return keys;
}

template<typename keyT, typename valT>
std::vector<valT> getValSetasVec(std::map<keyT, valT> in_map) {
	std::vector<valT> keys;
	std::transform(in_map.begin(), in_map.end(), std::back_inserter(keys),
			RetrieveVal());
	return keys;
}

template<typename A, typename B>
std::pair<B, A> flip_pair(const std::pair<A, B> &p) {
	return std::pair<B, A>(p.second, p.first);
}

template<typename A, typename B>
std::multimap<B, A> flip_map(const std::map<A, B> &src) {
	std::multimap<B, A> dst;
	std::transform(src.begin(), src.end(), std::inserter(dst, dst.begin()),
			flip_pair<A, B>);
	return dst;
}

// =============================
// ==== TO STRING FUNCTIONS ====
// =============================
//------------------------------------------------------------------------------------------------------

template<typename T>
const std::string toString(T value) {
	std::ostringstream os;
	os << value;
	return os.str();
}

template<typename T>
const std::string toString(std::set<T> myset) {
	std::ostringstream os;
	os << "{";
	for (auto entry : myset) {
		os << toString(entry) << ",";
	}
	if (myset.size() > 0)
		os.seekp(-1, os.cur);
	os << "}";
	return os.str();
}

template<typename T>
const std::string toString(std::unordered_set<T>* myset) {
	std::ostringstream os;
	os << "{";
	for (auto entry : *myset) {
		os << toString(entry) << ",";
	}
	if (myset->size() > 0)
		os.seekp(-1, os.cur);
	os << "}";
	return os.str();
}

template<typename T>
const std::string toString(std::unordered_set<T> myset) {
	std::ostringstream os;
	os << "{";
	for (auto entry : myset) {
		os << toString(entry) << ",";
	}
	if (myset.size() > 0)
		os.seekp(-1, os.cur);
	os << "}";
	return os.str();
}

template<typename T>
const std::string toString(std::set<T> *myset) {
	std::ostringstream os;
	os << "*{";
	for (auto entry : *myset) {
		os << toString(entry) << ",";
	}
	if (myset->size() > 0)
		os.seekp(-1, os.cur);
	os << "}";
	return os.str();
}

template<typename T>
const std::string toString(std::vector<T> myset) {
	std::ostringstream os;
	os << "[";
	for (auto entry : myset) {
		os << toString(entry) << ",";
	}
	if (myset.size() > 0)
		os.seekp(-1, os.cur);
	os << "]";
	return os.str();
}

template<typename T>
const std::string toString(std::vector<T>* myset) {
	std::ostringstream os;
	os << "*[";
	for (auto entry : *myset) {
		os << toString(entry) << ",";
	}
	if (myset->size() > 0)
		os.seekp(-1, os.cur);
	os << "]";
	return os.str();
}

template<typename T, typename D>
const std::string toString(std::map<T, D> myMap) {
	std::ostringstream os;
	os << "{";
	for (auto entry : myMap) {
		os << "(" << toString(entry.first) << ":" << toString(entry.second)
				<< ")";
	}
	if (myMap.size() > 0)
		os.seekp(-1, os.cur);
	os << "}";
	return os.str();
}

template<typename T, typename D>
const std::string toString(std::map<T, D>* myMap) {
	std::ostringstream os;
	os << "{";
	for (auto entry : (*myMap)) {
		os << "(" << toString(entry.first) << ":" << toString(entry.second)
				<< ")";
	}
//	if (myMap->size() > 0)
//		os.seekp(-1, os.cur);
	os << "}";
	return os.str();
}

inline const std::string toString(bool value) {
	return value ? "true" : "false";
}

inline const std::string toString(coordinate coord) {
	return "(" + toString(coord.x + 1) + "," + toString(coord.y + 1) + ","
			+ toString(coord.z + 1) + ")";
}

inline const std::string toString(std::string str) {
	return str;
}

std::string padStringAfter(std::string toPad, int length);

// ==========================
// ==== SYSTEM FUNCTIONS ====
// ==========================
//------------------------------------------------------------------------------------------------------

int getPeakMemoryUsage();

int getCurrentMemoryUsage();

/*
 * Get current date/time, format is YYYY-MM-DD.HH:mm:ss */
const std::string currentDateTime();

//System Level Helper Functions
bool folderExsits(std::string path);

//bool createFolder(std::string path) {
//	throw std::runtime_error("Have no cross platform way for this...");
//}

#endif /* UTILITY_ */
