//
//  libpBloomFilter.h
//  pBloomFilter
//
//  Created by Pauli Miettinen on 27.8.2012.
//  Copyright (c) 2012 Pauli Miettinen. All rights reserved.
//

#ifndef pBloomFilter_libpBloomFilter_h
#define pBloomFilter_libpBloomFilter_h

#define USE_BLOOM_OMP

/* Struct to hold the Bloom filter data */
typedef struct {
	unsigned int numBitsExp;
	unsigned long numBits;
	unsigned long bitMask;
	char *bArray;
	unsigned int capacity;
	double errorRate;
} mbf;

/* Creates a Bloom Filter capable of holding capacity bits with error rate error rate */
mbf *
bloom_create_mbf(unsigned int capacity, double errorRate);

/* Destroys a Bloom Filter */
void
bloom_destroy_mbf(mbf* bf);

/* Adds an item to mbf */
void
bloom_add_mbf(mbf * const bf, const int key1, const int key2, const int key3);

/* Functions to query the status of the Bloom filter */
unsigned int
bloom_get_numBitsExp(const mbf *);

unsigned long
bloom_get_numBits(const mbf *);

unsigned int
bloom_get_capacity(const mbf *);

double
bloom_get_errorRate(const mbf *);

/* Checks a value in mbf */
int
bloom_check_mbf(const mbf *bf, const int key1, const int key2, const int key3);

/* Checks multiple values of Bloom filter and returns the count of found objects */
/*returns if the number of zeros is above maxErr*/
int
bloom_multi_check_mbf_zeros(const mbf *bf, const int *key1, const int *key2,
		const int *key3, const int len, const int maxErr);

/* Checks multiple values of Bloom filter and returns the count of found objects */
/*returns if the number of ones is above minOnes. Used to count min density*/
int
bloom_multi_check_mbf_ones(const mbf *bf, const int *key1, const int *key2,
		const int *key3, const int len, const int minOnes);
/*
 * Checks the number of 1s in the filter using all the combinations of values in x, y, and z
 */
int
bloom_check_ones_product(const mbf *bf, const int *x, const int lenx,
		const int *y, const int leny, const int *z, const int lenz,
		const int minones);

/* Computes the overall error caused by the merge of two blocks */
int bloom_check_merge_mbf(const mbf *bf, const int *r1, // Rows of the first block
		const int *c1,  // Cols           "
		const int *t1,  // Tubes          "
		const int *lengths1, // Lengths of the above three
		const int *r2,  // Rows of the second block
		const int *c2,  // Cols           "
		const int *t2,  // Tubes          "
		const int *lengths2, // Lengths of the above three
		const int maxerr // Maximum error
		);

#endif
